@@ -1,23 +1,27 @@
 ;;; nyacc/lalr.scm
-;;;
-;;; Copyright (C) 2014-2018 Matthew R. Wette
-;;;
-;;; This library is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU Lesser General Public
-;;; License as published by the Free Software Foundation; either
-;;; version 3 of the License, or (at your option) any later version.
-;;;
-;;; This library is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; Lesser General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU Lesser General Public License
-;;; along with this library; if not, see <http://www.gnu.org/licenses/>
+
+;; Copyright (C) 2014-2018 Matthew R. Wette
+;;
+;; This library is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU Lesser General Public
+;; License as published by the Free Software Foundation; either
+;; version 3 of the License, or (at your option) any later version.
+;;
+;; This library is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; Lesser General Public License for more details.
+;;
+;; You should have received a copy of the GNU Lesser General Public License
+;; along with this library; if not, see <http://www.gnu.org/licenses/>
+
+;;; Notes:
 
 ;; I need to find way to preserve srconf, rrconf after hashify.
 ;; compact needs to deal with it ...
 
+;;; Code:
+
 (define-module (nyacc lalr)
   #:export (lalr-spec process-spec
 	    make-lalr-machine compact-machine hashify-machine 
@@ -33,6 +37,7 @@
   #:use-module (srfi srfi-43)
   #:use-module (nyacc util)
   #:use-module (nyacc version)
+  #:use-module (ice-9 pretty-print)
   )
 
 
@@ -193,9 +198,11 @@
     ((_) '())))
 
 (define-syntax lalr-spec-1
-  (syntax-rules (start expect notice reserve prec< prec> grammar)
+  (syntax-rules (start alt-start expect notice reserve prec< prec> grammar)
     ((_ (start <symb>) <e> ...)
      (cons (cons 'start '<symb>) (lalr-spec-1 <e> ...)))
+    ((_ (alt-start <sym1> <sym2> ...) <e> ...)
+     (cons (cons 'alt-start '(<sym1> <sym2> ...)) (lalr-spec-1 <e> ...)))
     ((_ (expect <n>) <e> ...)
      (cons (cons 'expect <n>) (lalr-spec-1 <e> ...)))
     ((_ (notice <str>) <e> ...)
@@ -342,16 +349,18 @@
   ;; TODO: which don't appear in OTHER RHS, e.g., (foo (foo))
   ;; @end deffn
   (define (gram-check-4 ll nl err-l)
-    (fold
-     (lambda (s l) (cons (fmtstr "+++ LHS not used in any RHS: ~A" s) l))
-     err-l
-     (let iter ((ull '()) (all ll)) ; unused LHSs, all LHS's
-       (if (null? all) ull
-	   (iter (if (or (memq (car all) nl)
-			 (memq (car all) ull)
-			 (eq? (car all) '$start))
-		     ull (cons (car all) ull))
-		 (cdr all))))))
+    (let ((alt-start (or (assq-ref tree 'alt-start) '())))
+      (fold
+       (lambda (s l) (cons (fmtstr "+++ LHS not used in any RHS: ~A" s) l))
+       err-l
+       (let iter ((ull '()) (all ll)) ; unused LHSs, all LHS's
+	 (if (null? all) ull
+	     (iter (if (or (memq (car all) nl)
+			   (memq (car all) ull)
+			   (memq (car all) alt-start) ; new 02Sep18
+			   (eq? (car all) '$start))
+		       ull (cons (car all) ull))
+		   (cdr all)))))))
 
   ;; TODO: check for repeated tokens in precedence spec's: prec<, prec>
 
@@ -1591,6 +1600,9 @@
 ;; It might be useful to add hashify and compact with keyword arguments.
 ;; @end deffn
 (define (make-lalr-machine spec)
+  "- Procedure: make-lalr-machine spec => pgen
+     Generate a-list of items used for building/debugging parsers.  It
+     might be useful to add hashify and compact with keyword arguments."
   (if (not spec) (error "make-lalr-machine: expecting valid specification"))
   (let ((prev-core (fluid-ref *lalr-core*)))
     (dynamic-wind
@@ -1812,6 +1824,10 @@
 			 (d (case ac
 			      ((shift) ds) ((reduce) (- ds))
 			      ((accept) 0) (else #f))))
+		    ;; If a rule is not used then ??? and then what? 180901
+		    ;;(cond
+		    ;; (t (iter2 (acons t d al1) (cdr al0)))
+		    ;; (else (iter2 (acons 0 0 al1) (cdr al0)))))))
 		    (unless t
 		      (fmterr "~S ~S ~S\n" tk ac ds)
 		      (error "expect something"))
@@ -1875,9 +1891,11 @@
 		    (list (string-append " " (elt->str e tl)))))))))))
 
 ;; @deffn {Procedure} pp-lalr-notice spec [port]
-;; @deffn pp-lalr-notice spec [port]
+;; This prints the text in the @code{notice} construct of a language spec.
 ;; @end deffn
 (define (pp-lalr-notice spec . rest)
+  "- Procedure: pp-lalr-notice spec [port]
+     This prints the text in the 'notice' construct of a language spec."
   (let* ((port (if (pair? rest) (car rest) (current-output-port)))
 	 (notice (assq-ref (assq-ref spec 'attr) 'notice))
 	 (lines (if notice (string-split notice #\newline) '())))
@@ -1888,6 +1906,8 @@
 ;; Pretty-print the grammar to the specified port, or current output.
 ;; @end deffn
 (define (pp-lalr-grammar spec . rest)
+  "- Procedure: pp-lalr-grammar spec [port]
+     Pretty-print the grammar to the specified port, or current output."
   (let* ((port (if (pair? rest) (car rest) (current-output-port)))
 	 (lhs-v (assq-ref spec 'lhs-v))
 	 (rhs-v (assq-ref spec 'rhs-v))
@@ -1916,6 +1936,8 @@
 ;; Print the states of the parser with items and shift/reduce actions.
 ;; @end deffn
 (define (pp-lalr-machine mach . rest)
+  "- Procedure: pp-lalr-machine mach [port]
+     Print the states of the parser with items and shift/reduce actions."
   (let* ((port (if (pair? rest) (car rest) (current-output-port)))
 	 (lhs-v (assq-ref mach 'lhs-v))
 	 (rhs-v (assq-ref mach 'rhs-v))
@@ -1989,9 +2011,6 @@
 
 ;; === output routines ===============
 
-(use-modules (ice-9 pretty-print))
-(use-modules (ice-9 regex))
-
 (define (write-notice mach port)
   (let* ((comm-leader ";; ")
 	 (notice (assq-ref (assq-ref mach 'attr) 'notice))
@@ -2001,11 +2020,10 @@
      lines)
     (if (pair? lines) (newline port))))
 
-(define (string-sub str pat repl)
-  (let ((m (string-match pat str)))
-    (if m
-	(regexp-substitute #f m 'pre repl 'post)
-	str)))
+(define (drop-dot-new filename)
+  (if (string-suffix? ".new" filename)
+      (string-drop-right filename 4)
+      filename))
 
 ;; @deffn {Procedure} write-lalr-tables mach filename [optons]
 ;; Options are
@@ -2037,12 +2055,18 @@
 
   (call-with-output-file filename
     (lambda (port)
-      (fmt port ";; ~A\n\n" (string-sub filename ".new$" ""))
+      (fmt port ";; ~A\n\n" (drop-dot-new filename))
       (write-notice mach port)
       (write-table mach 'len-v port)
       (write-table mach 'pat-v port)
       (write-table mach 'rto-v port)
       (write-table mach 'mtab port)
+      ;; generate alist
+      (fmt port "(define ~Atables\n  (list\n" prefix)
+      (fmt port "   (cons 'len-v ~Alen-v)\n" prefix)
+      (fmt port "   (cons 'pat-v ~Apat-v)\n" prefix)
+      (fmt port "   (cons 'rto-v ~Arto-v)\n" prefix)
+      (fmt port "   (cons 'mtab ~Amtab)))\n\n" prefix)
       (display ";;; end tables" port)
       (newline port))))
 
@@ -2100,7 +2124,7 @@
 
   (call-with-output-file filename
     (lambda (port)
-      (fmt port ";; ~A\n\n" (string-sub filename ".new$" ""))
+      (fmt port ";; ~A\n\n" (drop-dot-new filename))
       (write-notice mach port)
       (write-actions mach port)
       (display ";;; end tables" port)
