@@ -68,8 +68,8 @@
      (constant ($$ `(p-expr ,$1)))
      (string-literal ($$ `(p-expr ,$1)))
      ("(" expression ")" ($$ $2))
-     ("(" "{" block-item-list "}" ")"
-      ($$ `(stmt-expr (@ (extension "GNUC")) ,$3)))
+     ("(" "{" ($$ (cpi-push)) block-item-list ($$ (cpi-pop)) "}" ")"
+      ($$ `(stmt-expr (@ (extension "GNUC")) ,$4)))
      )
 
     (postfix-expression			; S 6.5.2
@@ -219,9 +219,9 @@
     (declaration-no-comment
      (declaration-specifiers
       init-declarator-list
-      ($$ (save-typenames `(decl ,$1 ,$2))))
-     (declaration-specifiers
-      ($$ (if (only-attr-specs? $1) `(expr-stmt) `(decl ,$1)))))
+      ($$ (save-typenames `(decl ,$1 ,$2)))) ;; move-attributes?
+     (declaration-specifiers ;; FIXME: need to trap attributes ???
+      ($$ `(decl ,$1))))
 
     (declaration-specifiers		; S 6.7
      (declaration-specifiers-1 ($$ (tl->list $1))))
@@ -243,7 +243,9 @@
       ($prec 'reduce-on-attr) ($$ (make-tl 'decl-spec-list $1)))
      (function-specifier declaration-specifiers-1 ($$ (tl-insert $2 $1)))
      ;; attributes
-     (attribute-specifier declaration-specifiers-1 ($$ (tl-insert $2 $1))))
+     ;;(attribute-specifier declaration-specifiers-1 ($$ (tl-insert $2 $1))))
+     (attribute-specifier declaration-specifiers-1 ($$ $2))
+     )
 
     (init-declarator-list		; S 6.7
      (init-declarator-list-1 ($$ (tl->list $1))))
@@ -424,9 +426,9 @@
      ("__attribute__" "(" "(" attribute-list ")" ")" ($$ (tl->list $4)))
      (attr-name ($$ `(attributes ,$1))))
     (attr-name
-     ("__packed__" ($$ '(ident "packed")))
-     ("__aligned__" ($$ '(ident "aligned")))
-     ("__alignof__" ($$ '(ident "alignof"))))
+     ("__packed__" ($$ '(ident "__packed__")))
+     ("__aligned__" ($$ '(ident "__aligned__")))
+     ("__alignof__" ($$ '(ident "__alignof__"))))
 
     (attribute-list
      (attribute ($$ (make-tl 'attributes $1)))
@@ -628,8 +630,8 @@
      ("default" ":" statement ($$ `(default ,$3))))
 
     (compound-statement
-     ("{" block-item-list "}" 
-      ($$ `(compd-stmt ,(tl->list $2))))
+     ("{" ($$ (cpi-push)) block-item-list ($$ (cpi-pop)) "}" 
+      ($$ `(compd-stmt ,(tl->list $3))))
      ("{" "}"
       ($$ `(compd-stmt (block-item-list)))))
 
@@ -736,8 +738,9 @@
      (declaration-specifiers
       declarator declaration-list compound-statement
       ($$ `(knr-fctn-defn ,(tl->list $1) ,$2 ,(tl->list $3) ,$4)))
-     (declaration-specifiers declarator compound-statement
-			     ($$ `(fctn-defn ,$1 ,$2 ,$3))))
+     (declaration-specifiers
+      declarator compound-statement
+      ($$ `(fctn-defn ,$1 ,$2 ,$3))))
     
     (declaration-list
      (declaration ($$ (make-tl $1)))
